@@ -1,4 +1,5 @@
 import six
+import sys
 import time
 
 from . import exceptions
@@ -13,31 +14,31 @@ class Socket(object):
     def __init__(self, server, sid):
         self.server = server
         self.sid = sid
-        self.queue = self.create_queue()
+        self.queue = self.server.create_queue()
         self.last_ping = time.time()
         self.connected = False
+        self.upgrading = False
         self.upgraded = False
+        self.packet_backlog = []
         self.closing = False
         self.closed = False
-
-    def create_queue(self):
-        return getattr(self.server._async['queue'],
-                       self.server._async['queue_class'])()
+        self.session = {}
 
     def poll(self):
         """Wait for packets to send to the client."""
         try:
             packets = [self.queue.get(timeout=self.server.ping_timeout)]
             self.queue.task_done()
-        except self.server._async['queue'].Empty:
+        except self.queue.Empty:
             raise exceptions.QueueEmpty()
         if packets == [None]:
             return []
-        try:
-            packets.append(self.queue.get(block=False))
-            self.queue.task_done()
-        except self.server._async['queue'].Empty:
-            pass
+        while True:
+            try:
+                packets.append(self.queue.get(block=False))
+                self.queue.task_done()
+            except self.queue.Empty:
+                break
         return packets
 
     def receive(self, pkt):
@@ -53,7 +54,7 @@ class Socket(object):
             self.send(packet.Packet(packet.PONG, pkt.data))
         elif pkt.packet_type == packet.MESSAGE:
             self.server._trigger_event('message', self.sid, pkt.data,
-                                       async=self.server.async_handlers)
+                                       run_async=self.server.async_handlers)
         elif pkt.packet_type == packet.UPGRADE:
             self.send(packet.Packet(packet.NOOP))
         elif pkt.packet_type == packet.CLOSE:
@@ -61,16 +62,31 @@ class Socket(object):
         else:
             raise exceptions.UnknownPacketError()
 
-    def send(self, pkt):
-        """Send a packet to the client."""
+    def check_ping_timeout(self):
+        """Make sure the client is still sending pings.
+
+        This helps detect disconnections for long-polling clients.
+        """
         if self.closed:
-            raise IOError('Socket is closed')
-        if time.time() - self.last_ping > self.server.ping_timeout:
+            raise exceptions.SocketIsClosedError()
+        if time.time() - self.last_ping > self.server.ping_interval + 5:
             self.server.logger.info('%s: Client is gone, closing socket',
                                     self.sid)
-            self.close(wait=False, abort=True)
+            # Passing abort=False here will cause close() to write a
+            # CLOSE packet. This has the effect of updating half-open sockets
+            # to their correct state of disconnected
+            self.close(wait=False, abort=False)
+            return False
+        return True
+
+    def send(self, pkt):
+        """Send a packet to the client."""
+        if not self.check_ping_timeout():
             return
-        self.queue.put(pkt)
+        if self.upgrading:
+            self.packet_backlog.append(pkt)
+        else:
+            self.queue.put(pkt)
         self.server.logger.info('%s: Sending packet %s data %s',
                                 self.sid, packet.packet_names[pkt.packet_type],
                                 pkt.data if not isinstance(pkt.data, bytes)
@@ -90,8 +106,9 @@ class Socket(object):
         try:
             packets = self.poll()
         except exceptions.QueueEmpty:
+            exc = sys.exc_info()
             self.close(wait=False)
-            raise
+            six.reraise(*exc)
         return packets
 
     def handle_post_request(self, environ):
@@ -109,10 +126,11 @@ class Socket(object):
         """Close the socket connection."""
         if not self.closed and not self.closing:
             self.closing = True
-            self.server._trigger_event('disconnect', self.sid, async=False)
+            self.server._trigger_event('disconnect', self.sid, run_async=False)
             if not abort:
                 self.send(packet.Packet(packet.CLOSE))
             self.closed = True
+            self.queue.put(None)
             if wait:
                 self.queue.join()
 
@@ -120,13 +138,10 @@ class Socket(object):
         """Upgrade the connection from polling to websocket."""
         if self.upgraded:
             raise IOError('Socket has been upgraded already')
-        if self.server._async['websocket'] is None or \
-                self.server._async['websocket_class'] is None:
+        if self.server._async['websocket'] is None:
             # the selected async mode does not support websocket
             return self.server._bad_request()
-        websocket_class = getattr(self.server._async['websocket'],
-                                  self.server._async['websocket_class'])
-        ws = websocket_class(self._websocket_handler)
+        ws = self.server._async['websocket'](self._websocket_handler)
         return ws(environ, start_response)
 
     def _websocket_handler(self, ws):
@@ -138,19 +153,20 @@ class Socket(object):
 
         if self.connected:
             # the socket was already connected, so this is an upgrade
+            self.upgrading = True  # hold packet sends during the upgrade
             self.queue.join()  # flush the queue first
 
             pkt = ws.wait()
-            if pkt != packet.Packet(packet.PING,
-                                    data=six.text_type('probe')).encode(
-                                        always_bytes=False):
+            decoded_pkt = packet.Packet(encoded_packet=pkt)
+            if decoded_pkt.packet_type != packet.PING or \
+                    decoded_pkt.data != 'probe':
                 self.server.logger.info(
                     '%s: Failed websocket upgrade, no PING packet', self.sid)
                 return []
             ws.send(packet.Packet(
                 packet.PONG,
                 data=six.text_type('probe')).encode(always_bytes=False))
-            self.send(packet.Packet(packet.NOOP))
+            self.queue.put(packet.Packet(packet.NOOP))  # end poll
 
             pkt = ws.wait()
             decoded_pkt = packet.Packet(encoded_packet=pkt)
@@ -162,6 +178,12 @@ class Socket(object):
                     self.sid, pkt)
                 return []
             self.upgraded = True
+
+            # flush any packets that were sent during the upgrade
+            for pkt in self.packet_backlog:
+                self.queue.put(pkt)
+            self.packet_backlog = []
+            self.upgrading = False
         else:
             self.connected = True
             self.upgraded = True
@@ -207,16 +229,19 @@ class Socket(object):
             pkt = packet.Packet(encoded_packet=p)
             try:
                 self.receive(pkt)
-            except exceptions.UnknownPacketError:
+            except exceptions.UnknownPacketError:  # pragma: no cover
                 pass
+            except exceptions.SocketIsClosedError:  # pragma: no cover
+                self.server.logger.info('Receive error -- socket is closed')
+                break
             except:  # pragma: no cover
                 # if we get an unexpected exception we log the error and exit
                 # the connection properly
-                self.server.logger.exception('Receive error')
+                self.server.logger.exception('Unknown receive error')
                 break
 
         self.queue.put(None)  # unlock the writer task so that it can exit
         writer_task.join()
-        self.close(wait=True, abort=True)
+        self.close(wait=False, abort=True)
 
         return []
